(function() {
  var afterPerformRetry, beforePerformRetry, decorate, delayedPush, enqueueAt, enqueueIn, getJobWithRetry, getRegularJobsObject, getRetryAttempt, getRetryDelay, getRetryLimit, identifier, inspect, overrideFail, puts, redisRetryKey, retryFailure, tryAgain, _ref;
  var __indexOf = Array.prototype.indexOf || function(item) {
    for (var i = 0, l = this.length; i < l; i++) {
      if (this[i] === item) return i;
    }
    return -1;
  };
  _ref = require('sys'), puts = _ref.puts, inspect = _ref.inspect;
  exports.watcher = require('./scheduled-task-watcher');
  exports.createWorker = function(resque, queue, jobsWithRetry) {
    var worker;
    worker = resque.worker(queue, getRegularJobsObject(jobsWithRetry));
    decorate(worker, jobsWithRetry);
    return worker;
  };
  getRegularJobsObject = function(jobsWithRetry) {
    var job, jobs, name;
    jobs = {};
    for (name in jobsWithRetry) {
      job = jobsWithRetry[name];
      if (typeof job.func === 'function') {
        jobs[name] = job.func;
      }
    }
    return jobs;
  };
  decorate = function(worker, jobsWithRetry) {
    worker.jobsWithRetry = jobsWithRetry;
    worker.fail = overrideFail(worker);
    worker.on('job', function(worker, queue, job) {
      return beforePerformRetry(worker, job);
    });
    return worker.on('success', function(worker, queue, job, result) {
      return afterPerformRetry(worker, job);
    });
  };
  overrideFail = function(worker) {
    var originalFailFunc, redis;
    originalFailFunc = worker.fail;
    redis = worker.redis;
    return function(error, job) {
      return getRetryAttempt(worker, job, function(err, retryAttempt) {
        if (err == null) {
          if (retryAttempt < getRetryLimit(worker, job)) {
            return retryFailure(worker, job, err);
          } else {
            redis.del(redisRetryKey(job));
            return originalFailFunc.apply(worker, [error, job]);
          }
        }
      });
    };
  };
  getJobWithRetry = function(worker, job) {
    var jobDef, name, _ref;
    _ref = worker.jobsWithRetry;
    for (name in _ref) {
      jobDef = _ref[name];
      if (name === job["class"]) {
        return jobDef;
      }
    }
  };
  getRetryLimit = function(worker, job) {
    var _ref;
    return ((_ref = getJobWithRetry(worker, job)) != null ? _ref.retry_limit : void 0) || 0;
  };
  getRetryDelay = function(worker, job) {
    var _ref;
    return ((_ref = getJobWithRetry(worker, job)) != null ? _ref.retry_delay : void 0) || 0;
  };
  getRetryAttempt = function(worker, job, cb) {
    return worker.redis.get(redisRetryKey(job), cb);
  };
  identifier = function(args) {
    var a, types;
    types = ['string', 'number', 'boolean'];
    return ((function() {
      var _i, _len, _ref, _results;
      _results = [];
      for (_i = 0, _len = args.length; _i < _len; _i++) {
        a = args[_i];
        if (_ref = typeof a, __indexOf.call(types, _ref) >= 0) {
          _results.push(a.toString().slice(0, 40));
        }
      }
      return _results;
    })()).slice(0, 4).join('-');
  };
  redisRetryKey = function(job) {
    var args, name;
    name = job["class"];
    args = job.args;
    return ['resque-retry', name].concat(identifier(args)).join(":").replace(/\s/g, '');
  };
  beforePerformRetry = function(worker, job) {
    var key, redis;
    redis = worker.redis;
    key = redisRetryKey(job);
    redis.setnx(key, -1);
    return redis.incr(key);
  };
  retryFailure = function(worker, job, err) {
    var redis;
    redis = worker.redis;
    return getRetryAttempt(worker, job, function(err, retryAttempt) {
      if (err == null) {
        if (retryAttempt < getRetryLimit(worker, job)) {
          return tryAgain(redis, worker, job);
        } else {
          return redis.del(redisRetryKey(job));
        }
      }
    });
  };
  tryAgain = function(redis, worker, job) {
    var retryDelay;
    retryDelay = getRetryDelay(worker, job);
    if (retryDelay <= 0) {
      return require('coffee-resque').connect({
        redis: redis
      }).enqueue(worker.queue, job["class"], job.args);
    } else {
      return enqueueIn(redis, retryDelay, worker.queue, job["class"], job.args);
    }
  };
  afterPerformRetry = function(worker, job) {
    return worker.redis.del(redisRetryKey(job));
  };
  enqueueIn = function(redis, numberOfSecondsFromNow, queue, jobName, args) {
    var timestamp;
    timestamp = (new Date).valueOf() + numberOfSecondsFromNow * 1000;
    return enqueueAt(redis, timestamp, queue, jobName, args);
  };
  enqueueAt = function(redis, timestamp, queue, jobName, args) {
    return delayedPush(redis, timestamp, {
      queue: queue,
      "class": jobName,
      args: args
    });
  };
  delayedPush = function(redis, timestamp, item) {
    redis.rpush("delayed:" + timestamp, JSON.stringify(item));
    return redis.zadd('delayed_queue_schedule', timestamp, timestamp);
  };
}).call(this);
