(function() {
  var cleanUpTimestamp, enqueueDelayedItemsForTimestamp, enqueueScheduledTask, handleDelayedItems, inspect, intervalID, nextDelayedTimestamp, nextItemForTimestamp, puts, redis, stopping, _ref;
  _ref = require('sys'), puts = _ref.puts, inspect = _ref.inspect;
  redis = intervalID = null;
  stopping = false;
  exports.start = function(opts) {
    if (!((redis != null) && !stopping)) {
      redis = require('redis').createClient(opts.port, opts.host);
      return intervalID = setInterval(handleDelayedItems, 1000);
    }
  };
  exports.stop = function() {
    stopping = true;
    if (redis != null) {
      clearInterval(intervalID);
      redis.quit();
      return redis = intervalID = null;
    }
  };
  handleDelayedItems = function() {
    if (!stopping) {
      nextDelayedTimestamp(function(err, timestamp) {
        if (timestamp != null) {
          return enqueueDelayedItemsForTimestamp(timestamp, function(err) {
            if (err == null) {
              return nextDelayedTimestamp(arguments.callee);
            }
          });
        }
      });
    }
  };
  nextDelayedTimestamp = function(cb) {
    try {
      return redis.zrangebyscore('delayed_queue_schedule', '-inf', Date.now(), 'limit', 0, 1, function(err, timestamps) {
        return cb(err, timestamps != null ? timestamps[0] : void 0);
      });
    } catch (e) {
      if (!stopping) {
        throw e;
      }
    }
  };
  enqueueDelayedItemsForTimestamp = function(timestamp, cb) {
    return nextItemForTimestamp(timestamp, function(err, item) {
      if (item != null) {
        enqueueScheduledTask(item);
        return nextItemForTimestamp(timestamp, arguments.callee);
      } else {
        return cb(err);
      }
    });
  };
  nextItemForTimestamp = function(timestamp, cb) {
    var key;
    key = "delayed:" + timestamp;
    return redis.lpop(key, function(err, item) {
      cleanUpTimestamp(key, timestamp);
      return cb(err, item != null ? JSON.parse(item) : void 0);
    });
  };
  cleanUpTimestamp = function(key, timestamp) {
    return redis.llen(key, function(err, length) {
      if (length === 0) {
        redis.del(key);
        return redis.zrem('delayed_queue_schedule', timestamp);
      }
    });
  };
  enqueueScheduledTask = function(item) {
    return require('coffee-resque').connect({
      redis: redis
    }).enqueue(item.queue, item["class"], item.args);
  };
}).call(this);
